from django.apps import AppConfig as DjangoApponfig


class AppConfig(DjangoApponfig):
    name = "edc_refusal"
    verbose_name = "Edc Refusal"
    include_in_administration_section = True
    has_exportable_data = True
    default_auto_field = "django.db.models.BigAutoField"



from django.db import models
from edc_model.models import BaseUuidModel
from edc_utils import get_utcnow


class SubjectScreening(BaseUuidModel):
    screening_identifier = models.CharField(max_length=25, unique=True)

    report_datetime = models.DateTimeField(default=get_utcnow)

    age_in_years = models.IntegerField()

    eligible = models.BooleanField(default=True)

    refused = models.BooleanField(default=True)



from edc_auth.site_auths import site_auths
from edc_screening.auth_objects import SCREENING, SCREENING_SUPER

from .auth_objects import codenames

site_auths.update_group(*codenames, name=SCREENING, no_delete=True)
site_auths.update_group(*codenames, name=SCREENING_SUPER)



from django.db import models
from edc_model.models import OtherCharField
from edc_utils import get_utcnow


class SubjectRefusalModelMixin(models.Model):
    screening_identifier = models.CharField(max_length=50, unique=True)

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    reason = models.ForeignKey(
        "edc_refusal.RefusalReasons",
        on_delete=models.PROTECT,
        verbose_name="Reason for refusal to join",
    )

    other_reason = OtherCharField()

    comment = models.TextField(
        verbose_name="Additional Comments",
        null=True,
        blank=True,
    )

    def __str__(self):
        return self.screening_identifier

    def natural_key(self):
        return (self.screening_identifier,)  # noqa

    @staticmethod
    def get_search_slug_fields():
        return ["screening_identifier"]

    class Meta:
        abstract = True



from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.contrib.sites.models import Site
from django.test import TestCase, override_settings
from edc_utils.date import get_utcnow

from edc_refusal.forms import SubjectRefusalForm
from edc_refusal.models import RefusalReasons, SubjectRefusal

from ...utils import get_subject_refusal_model, get_subject_refusal_model_cls
from ..models import SubjectScreening


class TestForms(TestCase):
    @staticmethod
    def get_data():
        refusal_reason = RefusalReasons.objects.all()[0]
        return {
            "screening_identifier": "12345",
            "report_datetime": get_utcnow(),
            "reason": refusal_reason,
            "other_reason": None,
            "comment": None,
            "site": Site.objects.get(id=settings.SITE_ID),
        }

    @override_settings(SUBJECT_REFUSAL_MODEL="edc_refusal.subjectrefusal")
    def test_model_funcs(self):
        self.assertEqual(get_subject_refusal_model(), "edc_refusal.subjectrefusal")
        self.assertEqual(get_subject_refusal_model_cls(), SubjectRefusal)

    @override_settings(SUBJECT_REFUSAL_MODEL="edc_refusal.subjectrefusal")
    def test_subject_refusal_ok(self):
        SubjectScreening.objects.create(
            screening_identifier="12345",
            report_datetime=get_utcnow() - relativedelta(days=1),
            age_in_years=25,
            eligible=True,
            refused=False,
        )
        form = SubjectRefusalForm(data=self.get_data(), instance=None)
        form.is_valid()
        self.assertEqual(form._errors, {})
        form.save()
        self.assertEqual(SubjectRefusal.objects.all().count(), 1)

    @override_settings(SUBJECT_REFUSAL_MODEL="edc_refusal.subjectrefusal")
    def test_add_subject_refusal_set_subject_screening_refused_true(self):
        subject_screening = SubjectScreening.objects.create(
            screening_identifier="12345",
            report_datetime=get_utcnow() - relativedelta(days=1),
            age_in_years=25,
            eligible=True,
            refused=False,
        )
        self.assertFalse(subject_screening.refused)

        form = SubjectRefusalForm(data=self.get_data(), instance=None)
        form.save()
        subject_screening.refresh_from_db()
        self.assertTrue(subject_screening.refused)

    @override_settings(SUBJECT_REFUSAL_MODEL="edc_refusal.subjectrefusal")
    def test_delete_subject_refusal_sets_subject_screening_refused_false(self):
        subject_screening = SubjectScreening.objects.create(
            screening_identifier="12345",
            report_datetime=get_utcnow() - relativedelta(days=1),
            age_in_years=25,
            eligible=True,
            refused=False,
        )
        self.assertFalse(subject_screening.refused)

        form = SubjectRefusalForm(data=self.get_data(), instance=None)
        form.save()
        subject_screening.refresh_from_db()
        self.assertTrue(subject_screening.refused)

        subject_refusal = SubjectRefusal.objects.get(
            screening_identifier=subject_screening.screening_identifier
        )
        subject_refusal.delete()
        subject_screening.refresh_from_db()
        self.assertFalse(subject_screening.refused)



from django.core.exceptions import ObjectDoesNotExist
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from edc_screening.utils import get_subject_screening_model_cls

from ..utils import get_subject_refusal_model


@receiver(
    post_save,
    weak=False,
    dispatch_uid="subject_refusal_on_post_save",
)
def subject_refusal_on_post_save(sender, instance, raw, created, **kwargs):
    """Updates `refused` field on SubjectScreening."""
    if not raw and sender._meta.label_lower == get_subject_refusal_model():
        try:
            obj = get_subject_screening_model_cls().objects.get(
                screening_identifier=instance.screening_identifier
            )
        except ObjectDoesNotExist:
            pass
        else:
            obj.refused = True
            obj.save(update_fields=["refused"])


@receiver(
    post_delete,
    weak=False,
    dispatch_uid="subject_refusal_on_post_delete",
)
def subject_refusal_on_post_delete(sender, instance, using, **kwargs):
    """Updates/resets `refused` field on SubjectScreening."""
    if sender._meta.label_lower == get_subject_refusal_model():
        try:
            obj = get_subject_screening_model_cls().objects.get(
                screening_identifier=instance.screening_identifier
            )
        except ObjectDoesNotExist:
            pass
        else:
            obj.refused = False
            obj.save(update_fields=["refused"])
